
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const DigitCollector_cxx_Version =
    "$Id: DigitCollector.cxx,v 1.2 2002/04/05 08:07:41 manish Exp $";

#include "global.h"
#include <cassert>

#include "DigitCollector.hxx"
#include "SystemInfo.hxx"
#include <Data.hxx>
#include <cpLog.h>
#include "RgDigitTimerEvent.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"

// TODO  - replace tick count with time class and delay correct seconds
// TODO - implement dial string matching

const unsigned int
    DigitCollector::MAX_PREV_IN_URLS_SIZE =
    10;

DigitCollector::DigitCollector(Sptr < Fifo < Sptr < SipProxyEvent > > >cpq)
{
    RgConfiguration *configuration = RgConfiguration::instance();

    string proxyServer = configuration->getProxyServer();

    string::size_type colonPos = proxyServer.find(":");
    destHostIpAddr = proxyServer.substr(0, colonPos);
    if (colonPos < string::npos)
    {
        sipPortNum = proxyServer.substr(proxyServer.rfind(":") + 1);
    }
    else
    {
        sipPortNum = "5060";
    }

    // added for pstn-ip-pstn call setup

    dial = "";
    isNewDigit = false;
    dialMethod = INCOMPLETE_DIAL;
    myDialedUrl = "";

    //initiate the inter-digit timeout timer
    interDigitTimeout = Sptr < DigitTimeout > (new DigitTimeout(cpq));
    assert(interDigitTimeout != 0);

    interDigitTimeout->setInitTimeout(configuration->getInitialDigitTimeout());
    interDigitTimeout->setInterDigitTimeout(configuration->getInterDigitTimeout());

    DialingPlanListType dp = configuration->getDialPlans();

    DialingPlanListType::iterator dpIter = dp.begin();
    while (dpIter != dp.end())
    {
        DialingPlan *dialPattern = new DialingPlan(*(*dpIter));

        addDialPlan(dialPattern);
        dpIter++;
    }

    SpeedDialMapType sd = configuration->getSpeedDials();

    SpeedDialMapType::iterator sdIter = sd.begin();
    while (sdIter != sd.end())
    {
        addSpeedDial(sdIter->first, sdIter->second);
        sdIter++;
    }

    callProcessingQueue = cpq;
}

void
DigitCollector::reset()
{
    dial = "";
    isNewDigit = false;
    dialMethod = INCOMPLETE_DIAL;
    myDialedUrl = "";

    //reset the inter-digit timeout timer
    interDigitTimeout->timerReset();
}

void
DigitCollector::clear()
{
    dial = "";
    isNewDigit = false;
    dialMethod = INCOMPLETE_DIAL;
    myDialedUrl = "";

    //reset the inter-digit timeout timer
    interDigitTimeout->timerCancel();
}

void
DigitCollector::addDigit(char digit)
{
    //cancel the previous timeout timer
    interDigitTimeout->timerCancel();

    dial += digit;
    isNewDigit = true;

    //start the inter-digit timeout timer for the next digit
    interDigitTimeout->timerStart();

    // Check if dialing complete
    DialMethodsType stage = findDialPlan();

    if (!(stage == INCOMPLETE_DIAL || stage == WRONG_DIAL))
    {
        // Notify call processing
        Sptr < RgDigitTimerEvent > timerEvent =
            new RgDigitTimerEvent(callProcessingQueue);
        assert(timerEvent != 0);
        timerEvent->callId = RgDevice::instance()->getCallId();
        callProcessingQueue->add(timerEvent);
    }
}

void
DigitCollector::setDialedUrl(const string & url)
{
    dialMethod = URL_DIAL;
    myDialedUrl = url;

    // Notify call processing
    Sptr < RgDigitTimerEvent > timerEvent = new RgDigitTimerEvent(callProcessingQueue);
    assert(timerEvent != 0);
    timerEvent->callId = RgDevice::instance()->getCallId();
    callProcessingQueue->add(timerEvent);
}

bool DigitCollector::isValidDial()
{

//    if ( !isNewDigit ) return false;

    dialMethod = findDialPlan();
//    isNewDigit = false;

    if (INCOMPLETE_DIAL == dialMethod || WRONG_DIAL == dialMethod)
    {
        return false;
    }
    else
    {
        interDigitTimeout->timerCancel();
        return true;
    }

}

bool DigitCollector::isDialTimeout()
{
    return interDigitTimeout->isInterDigitTimeout();
}

string DigitCollector::getDial()
{
    return isDialTimeout()? dial + string("T") : dial;
}

string DigitCollector::getUrl()
{
    string
        url;

    switch (dialMethod)
    {
        case SPEED_DIAL:
            url = getUrlFromSpeedDial();
            break;
        case SPECIAL_KEY_DIAL:
            url = getUrlFromSpecKeyDial();
            break;
        case NORMAL_DIAL:
            url = getUrlFromPhoneNumDial();
            break;
        case INTERNAL_IP_DIAL:
            url = getUrlFromIpAddr();
            break;
        case CALL_RETURN_DIAL:
            url = getCallReturnUrl();
            break;
        case INCOMPLETE_DIAL:
        case WRONG_DIAL:
            url = "";
            break;
        case URL_DIAL:
            url = myDialedUrl;
            break;
        default:
            url = "";
            break;
    }
    return url;
}

string DigitCollector::getUrlFromPhoneNumDial()
{
    //get the current 3 digits of the IP address.
    string
        url =
        "sip:";

    url += dial;
    url += string("@");

    url += destHostIpAddr;

    if (sipPortNum == "")
    {
        url += ":5060";
    }
    else
    {
        url += string(":");
        url += sipPortNum;
    }

    cpLog(LOG_DEBUG, "DigitCollector url is returning %s", url.c_str());

    dialedType = dial_phone;

    return url;
}

string DigitCollector::getUrlFromSpecKeyDial()
{
    string
        dialStr =
        dial;

    if (!dialStr.empty())
        dialStr.resize(dialStr.length() - 1);

    string
        url =
        "sip:";

    url += dialStr;
    url += string("@");
    url += destHostIpAddr;

    if (sipPortNum == "")
    {
        url += ":5060";
    }
    else
    {
        url += string(":");
        url += sipPortNum;
    }

    cpLog(LOG_DEBUG, "DigitCollector url is returning %s", url.c_str());

    dialedType = dial_phone;

    return url;
}

string DigitCollector::getUrlFromSpeedDial()
{
    regex_t
        re;

    //get the current 3 digits of the IP address.
    string
        phoneNum =
        "";                     // String retrieved from speedDialMap
    string
        url =
        "";                     // url that we return
    string
        userName =
        "";
    string
        password =
        "";
    string
        destination =
        "";
    string
        sipPort =
        "";

    SpeedDialMapType::iterator speedDialIt = speedDialMap.find(dial);
    if (speedDialIt == speedDialMap.end())
    {
        cpLog(LOG_ERR, "Speed_Dial: %s  not found.", dial.c_str());
        phoneNum = "0000000";
    }
    else
    {
        phoneNum = speedDialIt->second;
    }

    if (0 == regcomp(&re, "[^0-9]", REG_EXTENDED))
    {
        string::size_type colonPos1, colonPos2, atPos;

        colonPos1 = phoneNum.find(":");
        colonPos2 = phoneNum.rfind(":");
        atPos = phoneNum.find("@");

        if (colonPos1 == string::npos)
        {
            // No ':'

            if (atPos == string::npos)
            {
                // No '@'
                // In this case, do we need to make sure that destination
                // is only [0-9]* ?
                userName = phoneNum;
                password = "";
                destination = "@" + destHostIpAddr;
                sipPort = ":" + sipPortNum;

            }
            else
            {
                // We have an '@'
                userName = phoneNum.substr(0, atPos);
                password = "";
                destination = phoneNum.substr(atPos);
                sipPort = ":5060";
            }
        }
        else                            // We have at least one ':'
        {
            // In this case, we need an '@'
            if (atPos == string::npos)
            {
                cpLog(LOG_ERR, "Error: No @ in Speed_Dial, exiting...\n");
                return 0;
            }

            if (colonPos1 != colonPos2)
            {
                // We have a port & a password
//              cpLog( LOG_DEBUG, "We have a port & a password" );

                userName = phoneNum.substr(0, colonPos1);
                password = phoneNum.substr(colonPos1, atPos - colonPos1);
                destination = phoneNum.substr(atPos, colonPos2 - atPos);
                sipPort = phoneNum.substr(colonPos2);
            }
            else
            {
                // We have either a port, or a password
                if (colonPos1 < atPos)
                {
                    // We have a password, but no port
//                  cpLog( LOG_DEBUG, "We have a password, but no port" );

                    userName = phoneNum.substr(0, colonPos1);
                    password = phoneNum.substr(colonPos1, atPos - colonPos1);
                    destination = phoneNum.substr(atPos);
                    sipPort = ":5060";
                }
                else
                {
                    // We have a port, but no password
//                  cpLog( LOG_DEBUG, "We have a port, but no password" );

                    userName = phoneNum.substr(0, atPos);
                    password = "";
                    destination = phoneNum.substr(atPos, colonPos2 - atPos);
                    sipPort = phoneNum.substr(colonPos2);
                }
            }
        }
        // Set the SipUrl
        url = "sip:" + userName + password + destination + sipPort;

        if (atPos == string::npos)
        {
            dialedType = dial_phone;
        }
        else
        {
            dialedType = dial_ip;
        }
    }
    regfree(&re);

    cpLog(LOG_DEBUG, "DigitCollector url is returning %s", url.c_str());
    return url;
}

string DigitCollector::getUrlFromIpAddr()
{
    //get the current 3 digits of the IP address.
    string
        url =
        "sip:user@";
    string
        firstIPpart =
        theSystem.
        getFirstIPpart();

    url += firstIPpart;                 //returns the first three digits.
    //getdial returns the last IP part.
    url += ".";

    string
        dial2 =
        dial;

    dial2.erase(0, 1);

    const char *
        dialStr =
        dial2.
        c_str();

    if (dialStr[0] != '0')
        url += dial;
    else if (dialStr[1] != '0')
    {
        url += dialStr[1];
        url += dialStr[2];
    }
    else
        url += dialStr[2];

    if (sipPortNum == "")
    {
        url += ":5060";
    }
    else
    {
        url += string(":");
        url += sipPortNum;

    }

    dialedType = dial_ip;

    cpLog(LOG_DEBUG, "DigitCollector url is returning %s", url.c_str());

    return url;
}

///
string DigitCollector::getCallReturnUrl()
{
    string
        url =
        "";

    if (prevInUrls.size() > 0)
    {
        list < string >::iterator iter = prevInUrls.begin();
        url = *iter;

        if (url.find(dial_phone) != string::npos)
            dialedType = dial_phone;
        else
            dialedType = dial_ip;
    }

    cpLog(LOG_DEBUG, "DigitCollector url is returning %s", url.c_str());

    return url;
}

///
void
DigitCollector::addSpeedDial(const string & key, const string & phoneNum)
{
    speedDialMap[key] = phoneNum;
}

///
void
DigitCollector::addDialPlan(const DialingPlan * dialingPlan)
{
    if (dialingPlan == 0)
    {
        return;
    }

    for (DialingPlanListType::iterator iter = dialingPlanList.begin();
         iter != dialingPlanList.end(); iter++)
    {
        const DialingPlan *pDialingPlan = *iter;

        if (pDialingPlan != 0 && *pDialingPlan == *dialingPlan)
        {
            dialingPlanList.erase(iter);
            break;
        }
    }

    dialingPlanList.push_back(dialingPlan);

    unsigned int patternLen = dialingPlan->getDialPattern().length();

    if (patternLen > maxDialPatternLen)
    {
        maxDialPatternLen = patternLen;
    }
}

///
void
DigitCollector::flushDialPlan()
{
    DialingPlanListType::iterator iDialingPlanList = dialingPlanList.begin();

    while (iDialingPlanList != dialingPlanList.end())
    {
        const DialingPlan *dialingPlan = *iDialingPlanList;
        delete dialingPlan;

        ++iDialingPlanList;
    }
    dialingPlanList.clear();
}

///
DialMethodsType DigitCollector::findDialPlan()
{
    regex_t
        re;
    DialMethodsType
        method =
        INCOMPLETE_DIAL;

    if (dialMethod == URL_DIAL)
    {
        return URL_DIAL;
    }

    //if the number of digits already dialed is longer than max length of all
    //dialing patterns we have, then we just return WRONG_DIAL
    if (dial.length() > maxDialPatternLen)
    {
        cpLog(LOG_DEBUG, "WRONG_DIAL-Number of dialed digits(%d) is too long",
              dial.length());
        return WRONG_DIAL;
    }

    //match the patterns.  We assume that there is only match in the list
    //of patterns we have
    DialingPlanListType::iterator iDialingPlanList = dialingPlanList.begin();

    cpLog(LOG_DEBUG, "Dialed string \"%s\"", dial.c_str());

    while (iDialingPlanList != dialingPlanList.end())
    {
        const DialingPlan *
            dialingPlan = *
            iDialingPlanList;

        if (!dialingPlan)
            continue;

        const char *
            pattern =
            dialingPlan->
            getDialPattern().
            c_str();

        if (0 != regcomp(&re, pattern, REG_EXTENDED))
        {
            cpLog(LOG_DEBUG, "WRONG_DIAL-Pattern not match");
            return WRONG_DIAL;
        }
        const char *
            dialstr =
            dial.
            c_str();

        if (REG_NOMATCH == regexec(&re, dialstr, 0, NULL, 0))
            ++iDialingPlanList;
        else
        {
            method = dialingPlan->getDialMethod();
            break;
        }
        regfree(&re);
    }

    switch (method)
    {
        case SPEED_DIAL:
            {
                cpLog(LOG_DEBUG, "SPEED_DIAL");
            }
            break;

        case SPECIAL_KEY_DIAL:
            {
                cpLog(LOG_DEBUG, "SPECIAL_KEY_DIAL");
            }
            break;

        case NORMAL_DIAL:
            {
                cpLog(LOG_DEBUG, "NORMAL_DIAL");
            }
            break;

        case INTERNAL_IP_DIAL:
            {
                cpLog(LOG_DEBUG, "INTERNAL_IP_DIAL");
            }
            break;

        case INCOMPLETE_DIAL:
            {
                cpLog(LOG_DEBUG, "INCOMPLETE_DIAL");
            }
            break;

        default:
            {
                cpLog(LOG_DEBUG, "WRONG_DIAL");
            }
    }                                   // switch

    return method;
}

//
void
DigitCollector::addIncomingUrl(const string & url)
{
    if (prevInUrls.size() >= MAX_PREV_IN_URLS_SIZE)
    {
        prevInUrls.pop_back();
    }
    prevInUrls.insert(prevInUrls.begin(), url);
}

///constructor for the dialing plan
DialingPlan::DialingPlan(string pattern, DialMethodsType method)
{
    dialPattern = pattern;
    dialMethod = method;
}

///
DialingPlan::DialingPlan(const DialingPlan & src)
{
    dialPattern = src.dialPattern;
    dialMethod = src.dialMethod;

}

///equality operator for DialingPlan.
///returns true if the dialPattern are equivalent
bool DialingPlan::operator == (const DialingPlan & dialPlan)
    const
    {
        return
            dialPattern ==
            dialPlan.
            dialPattern;
    }

///
    void
        DialingPlan::operator = (const DialingPlan & src){
    dialPattern = src.dialPattern;
    dialMethod = src.dialMethod;

    }
