
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpIP3xx.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <InviteMsg.hxx>
#include <AckMsg.hxx>
#include <StatusMsg.hxx>

#include "gwOpIP3xx.hxx"
#include "RgStateMachine.hxx"
#include "RgCallInfo.hxx"

#include "RgHardwareEvent.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"
#include "RgDeviceEvent.hxx"
#include "RgCallContainer.hxx"
#include "RgOperator.hxx"

///
gwOpIP3xx::gwOpIP3xx()
{
}

///
gwOpIP3xx::~gwOpIP3xx()
{
}

///
const char *const
gwOpIP3xx::name() const
{
    return "gwOpIP3xx";
}

//**************************************************************************
// gwOpIP3xx::process
//
// Description  : 
//      This is the operator for gwStateIPTrying. 
//      For a PSTN->IP call, it performs the following functions:
//      (1) Receive 3xx from IP user
//      (2) Stop the timer started in gwOpStartTImer.
//      (3) Send IP Busy to PSTN user.
//      (4) Send Release signal to PSTN user.
//      (5) Start timer for release.
//      (6) Change state to gwStateWait.
//**************************************************************************
///
const Sptr < State >
gwOpIP3xx::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpIP3xx operation");

    // check for 3xx from IP user
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);
    if (sipEvent == 0)
    {
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert(sipMsg != 0);

    Sptr < StatusMsg > msg;
    msg.dynamicCast(sipMsg);
    if (msg == 0)
    {
        return 0;
    }

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    if (*(call->findContact(*msg)) != *(call->getContact()))
    {
        cpLog(LOG_DEBUG, "Not the current call");
        return 0;
    }

    call->getContact()->update(*msg);

    int status = msg->getStatusLine().getStatusCode();

    if (status < 300 || status > 399)
    {
        cpLog(LOG_DEBUG, "Not Redirect Response. Status = %d", status);
        return 0;                       // Done
    }
    cpLog(LOG_DEBUG, "Received 3XX from IP user. Status = %d\n", status);
    AckMsg ack(*msg);

    sipEvent->getSipStack()->sendAsync(ack);

    // Stop the Timer started in gwOpStartTImer.
    Sptr < RgOperator > RgOpr;
    RgOpr->cancelTimer(event);
    cpLog(LOG_DEBUG, "Cancelling the response timer .");

    // start playing busy tone.... 
    cpLog(LOG_DEBUG, "Start Busy Tone");
    cout << "Start Busy Tone " << endl;

    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareSignalType;
    signal->signalOrRequest.signal = DeviceSignalBusyStart;

    RgDevice::getDeviceQueue()->add(signal);

    vusleep(5000000);                   // sleep for 5 sec

    // Send Release signal to PSTN user
    // i.e., send On Hook event on the PSTN line
    Sptr < RgHardwareEvent > signalOnHook =
        new RgHardwareEvent(RgDevice::getDeviceQueue());

    signalOnHook->type = HardwareSignalType;
    signalOnHook->signalOrRequest.signal = DeviceSignalPSTNOnHook;

    RgDevice::getDeviceQueue()->add(signalOnHook);

    // Start timer for release
    if (!RgOpr->
        setTimer(event, RgConfiguration::instance()->getRelTimeout(), "gwOpIP3xx"))
    {
        cpLog(LOG_DEBUG, "Release Timer not started");
    }
    else
    {
        cpLog(LOG_DEBUG, "Release Timer started");
    }

    // Change state to gwStateWait
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);
    cout << "WAIT" << endl;

    return stateMachine->findState("gwStateWait");

}
