
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpIPAnswered.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <AckMsg.hxx>

#include "RgHardwareEvent.hxx"
#include "RgStateMachine.hxx"
#include "RgCallInfo.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"
#include "RgOperator.hxx"

#include "gwOpIPAnswered.hxx"

///
gwOpIPAnswered::gwOpIPAnswered()
{
}

///
gwOpIPAnswered::~gwOpIPAnswered()
{
}

///
const char *const
gwOpIPAnswered::name() const
{
    return "gwOpIPAnswered";
}

//**************************************************************************
//      gwOpIPAnswered::process
//
//      Description     : 
//      This is the operator for gwStateIPTrying. 
//      For a PSTN->IP call, it performs the following functions:
//      (1) Receive the status msg for INVITE  (100 TRYING, 200 OK)
//      (2) Send ACK if 200 OK recieved
//      (3) Stop the Ringback tone
//      (4) Stop the timer started in IP ringing
//      (5) Change state to gwStateInCall
//
//**************************************************************************
const Sptr < State >
gwOpIPAnswered::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpIPAnswered operation");

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);
    if (sipEvent == 0)
    {
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert(sipMsg != 0);

    Sptr < StatusMsg > msg;
    msg.dynamicCast(sipMsg);
    if (msg == 0)
    {
        return 0;
    }

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    if (*(call->findContact(*msg)) != *(call->getContact()))
    {
        cpLog(LOG_DEBUG, "Not the current call");
        return 0;
    }

    call->getContact()->update(*msg);

    int status = msg->getStatusLine().getStatusCode();

    if (status >= 200)
    {
        // Needs to ack farend
        AckMsg ack(*msg);

        sipEvent->getSipStack()->sendAsync(ack);
    }

    if (status != 200)
    {
        return 0;                       // Done
    }

    // save the route
    call->setCallerRoute1List(msg->getrecordrouteList());
    int numContact = msg->getNumContact();

    if (numContact)
    {
        SipContact contact = msg->getContact(numContact - 1);

        Sptr < SipRoute > route = new SipRoute;
        route->setUrl(contact.getUrl());
        call->addRoute1(route);
    }

    // Stop the ringback tone
    cpLog(LOG_DEBUG, "Stop Ringback Tone");
    cout << "Stop Ringback Tone." << endl;

    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareSignalType;
    signal->signalOrRequest.signal = DeviceSignalLocalRingbackStop;

    RgDevice::getDeviceQueue()->add(signal);

    usleep(500);                        // 0.5 ms

    // Stop the Timer started in IP ringing
    Sptr < RgOperator > RgOpr;
    RgOpr->cancelTimer(event);
    cpLog(LOG_DEBUG, "gwOpIPAnswered:Timer is canceled here");

    // Get remote SDP from message
    Sptr < SipSdp > sdp;
    sdp.dynamicCast(sipMsg->getContentData(0));
    if (sdp == NULL)
    {
        cpLog(LOG_ERR, "No remote SDP in ACK");
    }
    else                                //if( sdp->isValidSdp() )
    {
        call->setRemoteSdp(new SipSdp(*sdp));
    }

    Sptr < SipSdp > localSdp = call->getLocalSdp();
    Sptr < SipSdp > remoteSdp = call->getRemoteSdp();

    // Change state to gwStateInCall
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    return stateMachine->findState("gwStateInCall");

}
