
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpPSTNStartCall.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <ctype.h>

#include <SipEvent.hxx>
#include <InviteMsg.hxx>

#include "global.h"
#include "RgCallInfo.hxx"
#include "RgDeviceEvent.hxx"
#include "RgDevice.hxx"
#include "RgStateMachine.hxx"

#include "gwOpPSTNStartCall.hxx"

///
gwOpPSTNStartCall::gwOpPSTNStartCall()
{
}

///
gwOpPSTNStartCall::~gwOpPSTNStartCall()
{
}

///
const char *const
gwOpPSTNStartCall::name() const
{
    return "gwOpPSTNStartCall";
}

//**************************************************************************
// gwOpPSTNStartCall::process
//
// Description  : 
// This is the operator for gwStateIdle. 
// For a IP-PSTN call, it performs the following functions:
// (1) Receive INVITE from IP user
// (2) Save call info in call container
// (3) Set local SDP
// (4) Send 100 Trying to IP user
// (5) Extract PSTN number from the Sip-To header of INVITE
// (6) Go Off Hook
// (7) Change the state to gwStatePSTNDialing
//**************************************************************************
const Sptr < State >
gwOpPSTNStartCall::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpPSTNStartCall operation");

    // Check whether a Sip Event
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);
    if (sipEvent == 0)
    {
        return 0;
    }

    // Check whether a Sip Msg (& not Status Msg)
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert(sipMsg != 0);

    // Check whether INVITE 
    Sptr < InviteMsg > msg;
    msg.dynamicCast(sipMsg);
    if (msg == 0)
    {
        cpLog(LOG_DEBUG, "Not an INVITE");
        return 0;
    }

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    // Save INVITE in Call info
    call->setRingInvite(new InviteMsg(*msg));
    // Set as current contact
    Sptr < Contact > contact = new Contact(*msg);
    call->setContact(contact);
    // Add to contact list
    call->addContact(contact);

    // save route info
    call->setCalleeRoute1List(msg->getrecordrouteList());
    int numContact = msg->getNumContact();

    if (numContact)
    {
        SipContact contact = msg->getContact(numContact - 1);

        Sptr < SipRoute > route = new SipRoute;
        route->setUrl(contact.getUrl());
        call->addRoute1(route);
    }

    // Associate the hardware with current INVITE
    // get the status of the hardware
    // available or not available

    Sptr < SipCallId > callId = new SipCallId(sipEvent->getSipCallLeg()->getCallId());
    HardwareStatusType hdwStatus = RgDevice::instance()->getHardwareStatus();

    if (hdwStatus == HARDWARE_AVAILABLE)
    {
        // Grab the device for this call
        cpLog(LOG_DEBUG, "RgDevice grabbed");
        RgDevice::instance()->setCallId(callId);
    }
    else
    {
        // this case should not happen
        // return 0;
    }

    cpLog(LOG_DEBUG, "Sending 100 Trying");
    StatusMsg status(*msg, 100 /*Trying */ );

    sipEvent->getSipStack()->sendReply(status);

    // Show caller information
    cout << VTime::strftime("%y/%m/%d %H:%M:%S")
    << " Call " << msg->getFrom().encode().getData();

    // Extract PSTN number from Sip-To of INVITE
    extern char pstnnum[20];

    strcpy(pstnnum, msg->getTo().getUrl()->getUserValue().convertString().c_str());
    cpLog(LOG_DEBUG, "PSTN number extracted from Sip-To %s", pstnnum);

    // go PSTN Off Hook
    Sptr < RgHardwareEvent > signalOffHook =
        new RgHardwareEvent(RgDevice::getDeviceQueue());

    signalOffHook->type = HardwareSignalType;
    signalOffHook->signalOrRequest.signal = DeviceSignalPSTNOffHook;

    RgDevice::getDeviceQueue()->add(signalOffHook);

    sleep(3);

    // Change state to gwStatePSTNDialing
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    cpLog(LOG_DEBUG, "Change state to gwStatePSTNDialing");
    return stateMachine->findState("gwStatePSTNDialing");
    //return 0;

}
