
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpStartAudioDuplex.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <cassert>

#include <cpLog.h>
#include <SipSdp.hxx>

#include "RgDevice.hxx"
#include "RgHardwareEvent.hxx"
#include "RgCallInfo.hxx"

#include "gwOpStartAudioDuplex.hxx"
#include "FormatSupportCheck.hxx"

///
gwOpStartAudioDuplex::gwOpStartAudioDuplex()
{
}

//**************************************************************************
// gwOpStartAudioDuplex::process
//
// Description  : 
// This is the operator for gwStateInCall. 
// It performs the following functions:
// (1)Set up the RTP path. 
//**************************************************************************
const Sptr < State >
gwOpStartAudioDuplex::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "StartAudioDuplex operation");

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    Sptr < SipSdp > remoteSdp;
    Sptr < SipSdp > localSdp;

    // get the current invite msg
    Sptr < InviteMsg > inviteMsg = call->getRingInvite();
    if (*inviteMsg == call->getContact()->getInviteMsg())
    {
        remoteSdp = call->getRemoteSdp();
        localSdp = call->getLocalSdp();
    }

    // if remote sdp is not avialable, don't start audio
    if (remoteSdp == 0)
    {
        cpLog(LOG_DEBUG, "No Remote Sdp.  Audio not started");
        return 0;
    }

    assert(localSdp != 0);

    // Create a Hardware Event of type AudioRequest &
    // post it to the Device Queue

    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareAudioType;

    struct HardwareAudioRequest *request = &(signal->signalOrRequest.request);

    request->type = AudioStart;

    // Set remote host and port
    strcpy(request->remoteHost, remoteSdp->getConnAddress().getData());
    request->remotePort = remoteSdp->getRtpPort();

    // Set local host and port
    request->localPort = localSdp->getRtpPort();
    strcpy(request->localHost, localSdp->getConnAddress().getData());

    //TODO Why echo cancellation always true?
    request->echoCancellation = true;

    // Set RTP packet size
    int rtpPacketSize = 20;     // Default( in ms )

    SdpSession sdpDesc = localSdp->getSdpDescriptor();

    vector < SdpMedia * >mediaList;
    mediaList = sdpDesc.getMediaList();
    vector < SdpMedia * >::iterator mediaIterator = mediaList.begin();
    MediaAttributes *mediaAttrib;

    mediaAttrib = (*mediaIterator)->getMediaAttributes();
    if (mediaAttrib != 0)
    {
        vector < ValueAttribute * >*valueAttribList = mediaAttrib->getValueAttributes();
        if (valueAttribList->size() == 0)
        {
            cpLog(LOG_DEBUG, "No value attribute in media");
        }
        vector < ValueAttribute * >::iterator attribIterator = valueAttribList->begin();
        while (attribIterator != valueAttribList->end())
        {
            char *attribName = (*attribIterator)->getAttribute();

            if (strcmp(attribName, "ptime") == 0)
            {
                rtpPacketSize = Data((*attribIterator)->getValue()).convertInt();
                break;
            }
            attribIterator++;
        }
        request->rtpPacketSize = rtpPacketSize;
    }
    else
    {
        cpLog(LOG_DEBUG, "No media attributes in local SDP");
    }

    cpLog(LOG_DEBUG, "Start audio( duplex ):");
    cpLog(LOG_DEBUG, "localHost     = %s", request->localHost);
    cpLog(LOG_DEBUG, "localPort     = %d", request->localPort);
    cpLog(LOG_DEBUG, "remoteHost    = %s", request->remoteHost);
    cpLog(LOG_DEBUG, "remotePort    = %d", request->remotePort);
    cpLog(LOG_DEBUG, "rtpPacketSize = %d", request->rtpPacketSize);

    //TODO: this will be removed once 183 is integrated.
    request->sendRingback = false;

    // Adding the network format information here.
    Sptr < Contact > contact = call->getContact();
    SdpSession sdpNetDesc;

    if (contact->getStatus() != 200)
    {
        //Iam a callee. Hence network format will be according to local sdp
        sdpNetDesc = localSdp->getSdpDescriptor();
    }
    else
    {
        //Iam a caller. Hence network format will be according to remote sdp
        sdpNetDesc = remoteSdp->getSdpDescriptor();
    }

    // declare the media list pointer 
    vector < SdpMedia * >mediaNetList;
    // get the media list pointer from the session object
    mediaNetList = sdpNetDesc.getMediaList();

    // declare the media list iterator and assign it to the first medial
    // object on the list

    vector < SdpMedia * >::iterator mediaNetIterator = mediaNetList.begin();
    while (mediaNetIterator != mediaNetList.end())
    {
        if ((*mediaNetIterator)->getMediaType() == MediaTypeAudio)
        {
            break;
        }
        ++mediaNetIterator;
    }

    if (mediaNetIterator == mediaNetList.end())
        return 0;                       // no audio media description available.

    vector < int >*formatList = (*mediaNetIterator)->getFormatList();
    int checkFormat;

    if (formatList != 0)
    {
        vector < int >::iterator fmtIterator = formatList->begin();

        while (fmtIterator != formatList->end())
        {
            checkFormat = *fmtIterator;

            if (FormatSupportCheck::instance()->FormatSupportedOnHardware(checkFormat))
            {
                break;
            }

            ++fmtIterator;
        }
        if (fmtIterator == formatList->end())
        {
            fmtIterator = formatList->begin();
            while (fmtIterator != formatList->end())
            {
                checkFormat = *fmtIterator;

                if (FormatSupportCheck::instance()->
                    FormatSupportedOnSoftware(checkFormat))
                {
                    break;
                }

                ++fmtIterator;
            }
            if (fmtIterator == formatList->end())
            {
                return 0;               // no format supported. Hence audio not started.
            }
        }
        request->networkFormat = (RtpPayloadType) checkFormat;
    }
    else
    {
        return 0;                       // No format supported on network. So audio not started.
    }

    RgDevice::getDeviceQueue()->add(signal);
    cout << "Call Established." << endl;
    return 0;
}

///
gwOpStartAudioDuplex::~gwOpStartAudioDuplex()
{
}

///
const char *const
gwOpStartAudioDuplex::name() const
{
    return "gwOpStartAudioDuplex";
}
